
--      astar.lua
--      
--      Copyright 2012 Michael Kosler <marekkpie@gmail.com>
--      
--      This program is free software; you can redistribute it and/or modify
--      it under the terms of the GNU General Public License as published by
--      the Free Software Foundation; either version 2 of the License, or
--      (at your option) any later version.
--      
--      This program is distributed in the hope that it will be useful,
--      but WITHOUT ANY WARRANTY; without even the implied warranty of
--      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--      GNU General Public License for more details.
--      
--      You should have received a copy of the GNU General Public License
--      along with this program; if not, write to the Free Software
--      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
--      MA 02110-1301, USA.
--      
--      

local function findNextNode(openSet)
	findNextStart = love.timer.getTime()
	local pos = 1
	for k,v in pairs(openSet) do
		if openSet[k].fScore < openSet[pos].fScore then
			pos = k
		end
	end
	findNextEnd = love.timer.getTime()
	return pos
end

local function squareGScoreCalc(i)
	if i == 1 or 3 or 6 or 8 then
		return 14
	else
		return 10
	end
end

local function debugNeighbors(n)
	print("Neighbors")
	for _,v in ipairs(n) do
		print(v)
	end
end

local function cleanPathMap(pathMap, openSet, closedSet)
	cleanUpStart = love.timer.getTime()
	for _,v in pairs(openSet) do
		pathMap[v.pathLoc].open = false
	end
	for _,v in pairs(closedSet) do
		pathMap[v.pathLoc].closed = false
	end
	cleanUpEnd = love.timer.getTime()
end

local function buildPath(closedSet, startPos)
	buildPathStart = love.timer.getTime()
	local path = {closedSet[#closedSet]}
	while path[#path].pathLoc ~= startPos do
		table.insert(path, closedSet[path[#path].pCloseLoc])
	end
	buildPathEnd = love.timer.getTime()
	aStarEnd = love.timer.getTime()
	return path
end

function startPathing(pathMap, startPos, exitPos)
	aStarStart = love.timer.getTime()
	pathMap[startPos].parent = pathMap[startPos]
	pathMap[startPos].gScore = 0
	pathMap[startPos].fScore =
		pathMap[startPos].gScore + pathMap[startPos].hScore
	pathMap[startPos].open = true
	local openSet = {pathMap[startPos]}
	local closedSet = {}
	local testNode
	
	mainLoopStart = love.timer.getTime()
	while #openSet > 0 do
		nextNodePos = findNextNode(openSet)
		testNode = openSet[nextNodePos]
		table.remove(openSet, nextNodePos)
		pathMap[testNode.pathLoc].open = false
		pathMap[testNode.pathLoc].closed = true
		table.insert(closedSet, testNode)
		if closedSet[#closedSet].pathLoc == exitPos then
			mainLoopEnd = love.timer.getTime()
			cleanPathMap(pathMap, openSet, closedSet)
			return buildPath(closedSet, startPos)
		end
		neighborStart = love.timer.getTime()
		for k,v in pairs(testNode.neighbors) do
			if not pathMap[v].closed then
				local tempGScore = testNode.gScore + testNode.distance[k]
				--~ if mapType == "square" then
					--~ tempGScore = testNode.gScore + squareGScoreCalc(k)
				--~ elseif mapType == "hexagonal" then
					--~ tempGScore = testNode.gScore + 10
				--~ end
				if not pathMap[v].open then
					pathMap[v].open = true
					pathMap[v].parent = testNode
					pathMap[v].pCloseLoc = #closedSet
					pathMap[v].gScore = tempGScore
					pathMap[v].fScore = 
						pathMap[v].hScore + tempGScore
					table.insert(openSet, pathMap[v])
				elseif tempGScore < pathMap[v].gScore then
					pathMap[v].parent = testNode
					pathMap[v].gScore = tempGScore
					pathMap[v].fScore = 
						pathMap[v].hScore + tempGScore
				end
			end
		end
		neighborEnd = love.timer.getTime()
	end
	return {}
end
